package com.pj.biz;

import java.util.List;

import com.pj.project.tb_car_black.TbCarBlack;
import com.pj.project.tb_car_black.TbCarBlackService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import com.pj.utils.sg.*;
import com.pj.utils.so.*;
import com.pj.project4sp.SP;

import com.pj.current.satoken.StpUserUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;


/**
 * Controller: tb_car_black -- 车辆黑名单
 *
 * @author qzyReal
 */
@RestController
@RequestMapping("/TbCarBlack/")
public class TbCarBlackController {

    /**
     * 底层 Service 对象
     */
    @Autowired
    TbCarBlackService tbCarBlackService;

    /**
     * 增
     */
    @RequestMapping("add")
    @SaCheckPermission(TbCarBlack.PERMISSION_CODE_ADD)
    @Transactional(rollbackFor = Exception.class)
    public AjaxJson add(TbCarBlack t) {
        tbCarBlackService.add(t);
        return AjaxJson.getSuccessData(t);
    }

    /**
     * 删
     */
    @RequestMapping("delete")
    @SaCheckPermission(TbCarBlack.PERMISSION_CODE_DEL)
    public AjaxJson delete(Long id) {
        tbCarBlackService.delete(id);
        return AjaxJson.getSuccess();
    }

    /**
     * 删 - 根据id列表
     */
    @RequestMapping("deleteByIds")
    @SaCheckPermission(TbCarBlack.PERMISSION_CODE_DEL)
    public AjaxJson deleteByIds() {
        List<Long> ids = SoMap.getRequestSoMap().getListByComma("ids", long.class);
        int line = SP.publicMapper.deleteByIds(TbCarBlack.TABLE_NAME, ids);
        return AjaxJson.getByLine(line);
    }

    /**
     * 改
     */
    @RequestMapping("update")
    @SaCheckPermission(TbCarBlack.PERMISSION_CODE_EDIT)
    public AjaxJson update(TbCarBlack t) {
        tbCarBlackService.update(t);
        return AjaxJson.getSuccess();
    }

    /**
     * 查 - 根据id
     */
    @RequestMapping("getById")
    public AjaxJson getById(Long id) {
        TbCarBlack t = tbCarBlackService.getById(id);
        return AjaxJson.getSuccessData(t);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    @RequestMapping("getList")
    public AjaxJson getList() {
        SoMap so = SoMap.getRequestSoMap();
        long deptId = StpUserUtil.getDeptId();
        if (deptId != StpUserUtil.ADMIN_DEPT_ID) {
            so.put("deptId", deptId);
        }
        List<TbCarBlack> list = tbCarBlackService.getList(so.startPage());
        return AjaxJson.getPageData(so.getDataCount(), list);
    }


}
