package com.pj.biz;

import java.util.List;

import cn.dev33.satoken.stp.StpUtil;
import com.pj.project.tb_person_filing.TbPersonFiling;
import com.pj.project.tb_person_filing.TbPersonFilingService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import com.pj.utils.sg.*;
import com.pj.utils.so.*;
import com.pj.project4sp.SP;

import com.pj.current.satoken.StpUserUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;


/**
 * Controller: tb_person_filing -- 人员备案
 *
 * @author qzyReal
 */
@RestController
@RequestMapping("/TbPersonFiling/")
public class TbPersonFilingController {

    /**
     * 底层 Service 对象
     */
    @Autowired
    TbPersonFilingService tbPersonFilingService;

    /**
     * 增
     */
    @RequestMapping("add")
    @SaCheckPermission(TbPersonFiling.PERMISSION_CODE_ADD)
    public AjaxJson add(TbPersonFiling t) {
        tbPersonFilingService.add(t);
        return AjaxJson.getSuccessData(t);
    }

    /**
     * 删
     */
    @RequestMapping("delete")
    @SaCheckPermission(TbPersonFiling.PERMISSION_CODE_DEL)
    public AjaxJson delete(Integer id) {
        tbPersonFilingService.delete(id);
        return AjaxJson.getSuccess();
    }

    /**
     * 删 - 根据id列表
     */
    @RequestMapping("deleteByIds")
    @SaCheckPermission(TbPersonFiling.PERMISSION_CODE_DEL)
    public AjaxJson deleteByIds() {
        List<Long> ids = SoMap.getRequestSoMap().getListByComma("ids", long.class);
        int line = SP.publicMapper.deleteByIds(TbPersonFiling.TABLE_NAME, ids);
        return AjaxJson.getByLine(line);
    }

    /**
     * 改
     */
    @RequestMapping("update")
    @SaCheckPermission(TbPersonFiling.PERMISSION_CODE_EDIT)
    public AjaxJson update(TbPersonFiling t) {
        tbPersonFilingService.update(t);
        return AjaxJson.getSuccess();
    }

    /**
     * 查 - 根据id
     */
    @RequestMapping("getById")
    public AjaxJson getById(Integer id) {
        TbPersonFiling t = tbPersonFilingService.getById(id);
        return AjaxJson.getSuccessData(t);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    @RequestMapping("getList")
    public AjaxJson getList() {
        SoMap so = SoMap.getRequestSoMap();
        long deptId = StpUserUtil.getDeptId();
        if (deptId != StpUserUtil.ADMIN_DEPT_ID) {
            so.put("deptId", deptId);
        }
        List<TbPersonFiling> list = tbPersonFilingService.getList(so.startPage());
        return AjaxJson.getPageData(so.getDataCount(), list);
    }


}
