package com.pj.biz;

import java.util.List;

import com.pj.project.tb_terminal.TbTerminal;
import com.pj.project.tb_terminal.TbTerminalService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import com.pj.utils.sg.*;
import com.pj.utils.so.*;
import com.pj.project4sp.SP;

import com.pj.current.satoken.StpUserUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;


/**
 * Controller: tb_terminal -- 设备管理
 *
 * @author qzyReal
 */
@RestController
@RequestMapping("/TbTerminal/")
public class TbTerminalController {

    /**
     * 底层 Service 对象
     */
    @Autowired
    TbTerminalService tbTerminalService;



    /**
     * 删
     */
    @RequestMapping("delete")
    @SaCheckPermission(TbTerminal.PERMISSION_CODE_DEL)
    public AjaxJson delete(Long id) {
        tbTerminalService.delete(id);
        return AjaxJson.getSuccess();
    }

    /**
     * 删
     */
    @RequestMapping("unbind")
    @SaCheckPermission(TbTerminal.PERMISSION_CODE_UNBIND)
    public AjaxJson unbind(Long id) {
        tbTerminalService.unbind(id);
        return AjaxJson.getSuccess();
    }

    /**
     * 删 - 根据id列表
     */
    @RequestMapping("deleteByIds")
    @SaCheckPermission(TbTerminal.PERMISSION_CODE_DEL)
    public AjaxJson deleteByIds() {
        List<Long> ids = SoMap.getRequestSoMap().getListByComma("ids", long.class);
        int line = SP.publicMapper.deleteByIds(TbTerminal.TABLE_NAME, ids);
        return AjaxJson.getByLine(line);
    }

    /**
     * 改
     */
    @RequestMapping("update")
    @SaCheckPermission(TbTerminal.PERMISSION_CODE_EDIT)
    public AjaxJson update(TbTerminal t) {
        tbTerminalService.update(t);
        return AjaxJson.getSuccess();
    }

    /**
     * 查 - 根据id
     */
    @RequestMapping("getById")
    public AjaxJson getById(Long id) {
        TbTerminal t = tbTerminalService.getById(id);
        return AjaxJson.getSuccessData(t);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    @RequestMapping("getList")
    public AjaxJson getList() {
        SoMap so = SoMap.getRequestSoMap();
        List<TbTerminal> list = tbTerminalService.getList(so.startPage());
        return AjaxJson.getPageData(so.getDataCount(), list);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    @RequestMapping("restart")
    @SaCheckPermission(TbTerminal.PERMISSION_CODE_REBOOT)
    public AjaxJson restart(String sn) {
        tbTerminalService.restart(sn);
        return AjaxJson.getSuccess();
    }

    @RequestMapping("clear")
    @SaCheckPermission(TbTerminal.PERMISSION_CODE_CLEAR)
    public AjaxJson clear(Long id) {
        tbTerminalService.clearData(id);
        return AjaxJson.getSuccess();
    }

    @RequestMapping("open")
    @SaCheckPermission(TbTerminal.PERMISSION_CODE_OPEN)
    public AjaxJson open(Long id) {
        tbTerminalService.open(id);
        return AjaxJson.getSuccess();
    }

    @RequestMapping("sync-time")
    @SaCheckPermission(TbTerminal.PERMISSION_CODE_SYNC_TIME)
    public AjaxJson syncTime(Long id) {
        tbTerminalService.asyncTime(id);
        return AjaxJson.getSuccess();
    }


}
