package com.pj.current;

import java.net.InetSocketAddress;
import java.text.SimpleDateFormat;
import java.util.Date;

import com.pj.current.config.MyConfig;
import com.pj.current.netty.command.config.CommandNettyServer;
import com.pj.current.netty.heart.HeartNettyServer;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;

/**
 * springboot启动之后 
 * @author kong
 *
 */
@Component
@Slf4j
public class SaPlusStartup implements CommandLineRunner {

	@Value("${spring.application.name:sa-plus}")
    private String applicationName;
	
	@Value("${server.port:8080}")
    private String port;

    @Value("${server.servlet.context-path:}")
    private String path;

    @Value("${spring.profiles.active:}")
    private String active;

    @Resource
    private MyConfig myConfig;


   
    @Override
    public void run(String... args) {
        InetSocketAddress commandAddress = new InetSocketAddress(myConfig.getIp(),myConfig.getCommandPort());
        CommandNettyServer commandNettyServer=  new CommandNettyServer(commandAddress);
        commandNettyServer.setDaemon(true);
        commandNettyServer.start();

        InetSocketAddress hearAddress = new InetSocketAddress(myConfig.getIp(),myConfig.getHeartPort());
        HeartNettyServer heartNettyServer=  new HeartNettyServer(hearAddress);
        heartNettyServer.setDaemon(true);
        heartNettyServer.start();
    }

	/**
	 * 返回系统当前时间的YYYY-MM-dd hh:mm:ss 字符串格式
	 */
	private static String getNow(){
		return new SimpleDateFormat("yyyy-MM-dd HH:mm:ss").format(new Date());
	}
    
}

