package com.pj.current.netty.command.handler.impl;

import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.pj.current.netty.command.constants.CommandType;
import com.pj.current.netty.command.handler.IHandler;
import com.pj.current.netty.command.handler.dto.WhitelistDTO;
import com.pj.current.netty.dto.OKMsg;
import com.pj.current.netty.dto.RequestPackageDTO;
import com.pj.current.task.TaskService;
import com.pj.project.tb_person_filing.TbPersonFiling;
import com.pj.project.tb_person_filing.TbPersonFilingService;
import com.pj.project.tb_person_filing.task.Base64ToImage;
import com.pj.project.tb_terminal.TbTerminal;
import com.pj.project.tb_terminal.TbTerminalService;
import io.netty.channel.ChannelHandlerContext;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * 上传人员信息
 */
@Service
@Slf4j
public class UploadUserHandler implements IHandler {
    @Resource
    private TbTerminalService tbTerminalService;
    @Resource
    private TbPersonFilingService tbPersonFilingService;
    @Resource
    private TaskService taskService;



    @Override
    public CommandType commandType() {
        return CommandType.UPLOAD_USER;
    }

    @Override
    public void handler(ChannelHandlerContext ctx, RequestPackageDTO packageDTO) {
        JSONObject params = JSONUtil.parseObj(packageDTO.getPayload().getParams());
        String sn = params.getStr("sn");
        TbTerminal tbTerminal = tbTerminalService.findBySN(sn);
        if (tbTerminal == null || tbTerminal.getDeptId() == null) {
            log.error("设备未绑定=========:{}", sn);
            return;
        }
        ctx.write(OKMsg.ok());
        ctx.flush();
        Long deptId = tbTerminal.getDeptId();
        WhitelistDTO uploadUserDTO = JSONUtil.toBean(params.getStr("users"), WhitelistDTO.class);
        TbPersonFiling personFiling = tbPersonFilingService.findByIdCardAndDeptId(uploadUserDTO.getIdentity_number(),deptId);
        if (personFiling == null) {
            Date now = new Date();
            personFiling = new TbPersonFiling();
            personFiling.setCreateTime(now)
                    .setUpdateTime(now)
                    .setPin(uploadUserDTO.getPin())
                    .setCreateBy("设备上传")
                    .setIdCard(uploadUserDTO.getIdentity_number())
                    .setDeptId(deptId)
                    .setName(uploadUserDTO.getName());
            tbPersonFilingService.save(personFiling);
            //转换图片
            taskService.addTask(new Base64ToImage(RandomUtil.randomNumbers(10),2000,personFiling.getId(),uploadUserDTO.getPhoto()));
        }
        List<TbTerminal> list = tbTerminalService.findByDeptId(deptId);
        //分发人员信息,不分发上传设备
        list.stream().filter(t -> !StrUtil.equals(t.getSn(), sn))
                .forEach(t -> {
                    tbTerminalService.setCommand(CommandType.ISSUED_USER_COMMAND.getFuncId(), params, t);
                    tbTerminalService.startHeartCommand(sn);
                });
    }
}
