package com.pj.project.sys_dict;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import com.pj.utils.sg.*;
import com.pj.utils.so.*;
import com.pj.project4sp.SP;

import com.pj.current.satoken.StpUserUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;


/**
 * Controller: sys_dict -- 字典管理
 *
 * @author qzyReal
 */
@RestController
@RequestMapping("/SysDict/")
public class SysDictController {

    /**
     * 底层 Service 对象
     */
    @Autowired
    SysDictService sysDictService;

    /**
     * 增
     */
    @RequestMapping("add")
    @Transactional(rollbackFor = Exception.class)
    public AjaxJson add(SysDict s) {
        sysDictService.add(s);
        return AjaxJson.getSuccessData(s);
    }

    /**
     * 删
     */
    @RequestMapping("delete")
    public AjaxJson delete(Integer id) {
        sysDictService.delete(id);
        return AjaxJson.getSuccess();
    }

    /**
     * 删 - 根据id列表
     */
    @RequestMapping("deleteByIds")
    public AjaxJson deleteByIds() {
        List<Long> ids = SoMap.getRequestSoMap().getListByComma("ids", long.class);
        int line = SP.publicMapper.deleteByIds(SysDict.TABLE_NAME, ids);
        return AjaxJson.getByLine(line);
    }

    /**
     * 改
     */
    @RequestMapping("update")
    public AjaxJson update(SysDict s) {
        sysDictService.update(s);
        return AjaxJson.getSuccess();
    }

    /**
     * 查 - 根据id
     */
    @RequestMapping("getById")
    public AjaxJson getById(Integer id) {
        SysDict s = sysDictService.getById(id);
        return AjaxJson.getSuccessData(s);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    @RequestMapping("getList")
    public AjaxJson getList() {
        SoMap so = SoMap.getRequestSoMap();
        List<SysDict> list = sysDictService.getList(so.startPage());
        return AjaxJson.getPageData(so.getDataCount(), list);
    }

    /**
     * 查集合 (整个表数据转化为tree结构返回)
     */
    @RequestMapping("getTree")
    public AjaxJson getTree() {
        // 获取记录
        SoMap so = SoMap.getRequestSoMap();
        List<SysDict> list = sysDictService.getList(so);
        // 转为tree结构，并返回
        List<SoMap> listMap = SoMap.getSoMapByList(list);
        List<SoMap> listTree = SoMap.listToTree(listMap, "id", "pid", "children");
        return AjaxJson.getPageData(Long.valueOf(listMap.size()), listTree);
    }


}
