package com.pj.project.tb_car_filing;

import java.util.Date;
import java.util.List;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.IdcardUtil;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.pj.current.global.BusinessException;
import com.pj.project.tb_car_black.TbCarBlack;
import com.pj.project.tb_car_black.TbCarBlackService;
import com.pj.project.tb_dept.TbDept;
import com.pj.project.tb_dept.TbDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.pj.utils.so.*;

import javax.annotation.Resource;

/**
 * Service: tb_car_filing -- 车辆备案
 *
 * @author qzyReal
 */
@Service
public class TbCarFilingService extends ServiceImpl<TbCarFilingMapper, TbCarFiling> implements IService<TbCarFiling> {

    /**
     * 底层 Mapper 对象
     */
    @Autowired
    TbCarFilingMapper tbCarFilingMapper;

    @Resource
    private TbDeptService tbDeptService;




    /**
     * 增
     */
    public void add(TbCarFiling t) {
        t.setCreateBy(StpUtil.getLoginIdAsString()).setCreateTime(new Date());
        handler(t);
        this.save(t);
    }

    private void handler(TbCarFiling t) {
        if (!IdcardUtil.isValidCard(t.getDriverIdCard())) {
            throw new BusinessException("身份证不正确");
        }
        TbDept tbDept = tbDeptService.getById(t.getDeptId());
        if (tbDept == null) {
            throw new BusinessException("请选择部门");
        }
        String carNo = t.getCarNo().toUpperCase();
        Date now = new Date();
        t.setDeptName(tbDept.getName()).setUpdateTime(now).setCarNo(carNo)
                .setUpdateBy(StpUtil.getLoginIdAsString());
        t.setCarNo(t.getCarNo().toUpperCase());
    }

    /**
     * 删
     */
    public void delete(Long id) {
        this.removeById(id);
    }

    /**
     * 改
     */
    public void update(TbCarFiling t) {
        handler(t);
        this.updateById(t);
    }

    /**
     * 查
     */
    public TbCarFiling getById(Long id) {
        return tbCarFilingMapper.getById(id);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    public List<TbCarFiling> getList(SoMap so) {
        return tbCarFilingMapper.getList(so);
    }


}
