package com.pj.project.tb_car_visit_record;

import java.io.File;
import java.util.Date;
import java.util.List;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.pj.current.config.SystemObject;
import com.pj.project.tb_person_visit_record.TbPersonVisitRecord;
import com.pj.project.tb_venues.TbVenues;
import com.pj.project.tb_venues.TbVenuesService;
import com.pj.project4sp.uploadfile.UploadConfig;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.pj.utils.so.*;

import javax.annotation.Resource;

/**
 * Service: tb_car_visit_record -- 车辆出入记录
 *
 * @author qzyReal
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class TbCarVisitRecordService extends ServiceImpl<TbCarVisitRecordMapper, TbCarVisitRecord> implements IService<TbCarVisitRecord> {

    /**
     * 底层 Mapper 对象
     */
    @Autowired
    TbCarVisitRecordMapper tbCarVisitRecordMapper;
    @Resource
    private TbVenuesService tbVenuesService;

    @Resource
    private UploadConfig uploadConfig;

    /**
     * 增
     */
    public void add(TbCarVisitRecord t) {
        Date now = new Date();
        TbVenues tbVenues = tbVenuesService.getById(t.getChannelId());
        t.setRecordType(1).setCreateTime(now).setCarNo(t.getCarNo().toUpperCase())
                .setUpdateTime(now).setDeptId(tbVenues.getDeptId()).setDeptName(tbVenues.getDeptName())
                .setChannel(tbVenues.getName());
        this.save(t);
    }

    /**
     * 删
     */
    public void delete(Long id) {
        this.removeById(id);
    }

    /**
     * 改
     */
    public void update(TbCarVisitRecord t) {
        Date now=new Date();
        t.setUpdateTime(new Date());
        TbVenues tbVenues = tbVenuesService.getById(t.getChannelId());
        t.setCarNo(t.getCarNo().toUpperCase())
                .setUpdateTime(now)
                .setDeptId(tbVenues.getDeptId())
                .setDeptName(tbVenues.getDeptName())
                .setChannel(tbVenues.getName());
        this.updateById(t);
    }

    /**
     * 查
     */
    public TbCarVisitRecord getById(Long id) {
        return super.getById(id);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    public List<TbCarVisitRecord> getList(SoMap so) {
        return tbCarVisitRecordMapper.getList(so);
    }


    public String export(SoMap soMap) {
        List<TbCarVisitRecord> list = this.getList(soMap);
        String flieTypeFolder = "/export/";
        String currDateFolder = DateUtil.today();
        String fileName = "carRecord-" + DateUtil.format(new Date(), "yyyyMMddHHmmss") + ".xlsx";
        String fileFolder = new File(uploadConfig.rootFolder).getAbsolutePath() +
                uploadConfig.httpPrefix + flieTypeFolder + currDateFolder + "/";
        if (!FileUtil.exist(fileFolder)) {
            FileUtil.mkdir(fileFolder);
        }
        ExcelWriter writer = ExcelUtil.getWriter(fileFolder + fileName);
        writer.addHeaderAlias("deptName", "部门");
        writer.addHeaderAlias("driverName", "联系人");
        writer.addHeaderAlias("driverContact", "联系号码");
        writer.addHeaderAlias("channel", "通道");
        writer.addHeaderAlias("carNo", "车牌");
        writer.addHeaderAlias("visitTime", "时间");
        writer.addHeaderAlias("typeStr", "补录/自动");
        writer.addHeaderAlias("directionStr", "入场/离场");
        writer.setOnlyAlias(true);
        writer.write(list, true);
        writer.close();
        return SystemObject.config.getDomain() + uploadConfig.httpPrefix + flieTypeFolder + currDateFolder + "/" + fileName;
    }
}
