package com.pj.project.tb_person_black;

import java.util.Collections;
import java.util.Date;
import java.util.List;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.pj.current.global.BusinessException;
import com.pj.current.netty.command.constants.CommandType;
import com.pj.current.netty.command.handler.dto.BlackPersonDTO;
import com.pj.project.tb_person_filing.TbPersonFiling;
import com.pj.project.tb_terminal.TbTerminal;
import com.pj.project.tb_terminal.TbTerminalService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.pj.utils.so.*;

import javax.annotation.Resource;

/**
 * Service: tb_person_black -- 人员黑名单
 *
 * @author qzyReal
 */
@Service
public class TbPersonBlackService extends ServiceImpl<TbPersonBlackMapper, TbPersonBlack> implements IService<TbPersonBlack> {

    /**
     * 底层 Mapper 对象
     */
    @Autowired
    TbPersonBlackMapper tbPersonBlackMapper;

    @Resource
    @Lazy
    private TbTerminalService tbTerminalService;


    /**
     * 增
     */
    public void add(TbPersonBlack t) {
        if (!IdcardUtil.isValidCard(t.getIdCard())) {
            throw new BusinessException("身份证不正确");
        }
        TbPersonBlack db = findByIdCardAndDeptId(t.getIdCard(), t.getDeptId());
        if (db != null) {
            throw new BusinessException("人员已经存在");
        }

        Date now = new Date();
        String loginId = StpUtil.getLoginIdAsString();
        t.setCreateTime(now).setCreateBy(loginId).setUpdateBy(loginId).setUpdateTime(now);
        this.save(t);
        // todo 下发黑名单信息
        issued(t);
    }

    /**
     * 删
     */
    public void delete(Long id) {
        TbPersonBlack db = this.getById(id);
        this.removeById(id);
        deleteTerminalBlack(db);
    }

    /**
     * 改
     */
    public void update(TbPersonBlack t) {
        if (!IdcardUtil.isValidCard(t.getIdCard())) {
            throw new BusinessException("身份证不正确");
        }
        TbPersonBlack idCardDb = findByIdCardAndDeptId(t.getIdCard(), t.getDeptId());
        if (idCardDb != null && !idCardDb.getId().equals(t.getId())) {
            throw new BusinessException("人员已存在");
        }
        Date now = new Date();
        String loginId = StpUtil.getLoginIdAsString();
        t.setUpdateBy(loginId).setUpdateTime(now);
        TbPersonBlack db = this.getById(t.getId());
        //改了身份证或者名字,部门，要把旧的人员信息清除，下发新的
        if (!StrUtil.equals(t.getIdCard(), db.getIdCard())
                || !StrUtil.equals(t.getName(), db.getName())
                || !t.getDeptId().equals(db.getDeptId())
        ) {
            deleteTerminalBlack(db);
            issued(t);
        }
        this.updateById(t);
    }

    /**
     * 删除设备黑名单
     *
     * @param t
     */
    public void deleteTerminalBlack(TbPersonBlack t) {
        List<TbTerminal> tbTerminalList = tbTerminalService.findByDeptId(t.getDeptId());
        JSONObject params = new JSONObject();
        params.putIfAbsent("identity_numbers", Collections.singleton(t.getIdCard()));
        tbTerminalList.forEach(tbTerminal -> {
            tbTerminalService.setCommand(CommandType.DELETE_BLACKLIST_COMMAND.getFuncId(), params, tbTerminal);
            tbTerminalService.startHeartCommand(tbTerminal.getSn());
        });

    }


    /**
     * 查
     */
    public TbPersonBlack getById(Long id) {
        return tbPersonBlackMapper.getById(id);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    public List<TbPersonBlack> getList(SoMap so) {
        return tbPersonBlackMapper.getList(so);
    }

    /**
     * 下发所有黑名单到某一个设备
     *
     * @param t
     */
    public void issued(TbTerminal t) {
        Long deptId = t.getDeptId();
        List<TbPersonBlack> list = this.findByDeptId(deptId);
        list.forEach(tbPersonBlack -> {
            JSONObject users = builderData(tbPersonBlack);
            tbTerminalService.setCommand(CommandType.ISSUED_BLACKLIST_COMMAND.getFuncId(), users, t);
        });
    }

    /**
     * 下发单个黑名单到部门所有设备
     *
     * @param tbPersonBlack
     */
    public void issued(TbPersonBlack tbPersonBlack) {
        Long deptId = tbPersonBlack.getDeptId();
        List<TbTerminal> terminals = tbTerminalService.findByDeptId(deptId);
        JSONObject users = builderData(tbPersonBlack);
        terminals.forEach(tbTerminal -> {
            tbTerminalService.setCommand(CommandType.ISSUED_BLACKLIST_COMMAND.getFuncId(), users, tbTerminal);
            tbTerminalService.startHeartCommand(tbTerminal.getSn());
        });
    }

    /**
     * 构建黑名单数据格式
     *
     * @param tbPersonBlack
     * @return
     */
    private JSONObject builderData(TbPersonBlack tbPersonBlack) {
        BlackPersonDTO dto = new BlackPersonDTO();
        dto.setIdentity_number(tbPersonBlack.getIdCard()).setName(tbPersonBlack.getName());
        JSONObject users = new JSONObject();
        users.putIfAbsent("users", dto);
        return users;
    }

    /**
     * 根据部门查找
     *
     * @param deptId
     * @return
     */
    private List<TbPersonBlack> findByDeptId(Long deptId) {
        QueryWrapper<TbPersonBlack> ew = new QueryWrapper<>();
        ew.eq("dept_id", deptId);
        return list(ew);
    }

    public TbPersonBlack findByIdCardAndDeptId(String idCard, Long deptId) {
        QueryWrapper<TbPersonBlack> ew = new QueryWrapper<>();
        ew.eq("dept_id", deptId);
        ew.eq("id_card", idCard);
        return getOne(ew);
    }
}
