package com.pj.project.tb_person_filing;

import java.io.File;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.codec.Base64;
import cn.hutool.core.codec.Base64Encoder;
import cn.hutool.core.img.ImgUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.pj.current.config.SystemObject;
import com.pj.current.global.BusinessException;
import com.pj.current.netty.command.constants.CommandType;
import com.pj.current.netty.command.handler.dto.WhitelistDTO;
import com.pj.current.task.TaskService;
import com.pj.project.tb_person_black.TbPersonBlack;
import com.pj.project.tb_terminal.TbTerminal;
import com.pj.project.tb_terminal.TbTerminalService;
import com.pj.project4sp.uploadfile.UploadConfig;
import com.sun.imageio.plugins.common.ImageUtil;
import org.apache.poi.ss.util.ImageUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.pj.utils.so.*;

import javax.annotation.Resource;

/**
 * Service: tb_person_filing -- 人员备案
 *
 * @author qzyReal
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class TbPersonFilingService extends ServiceImpl<TbPersonFilingMapper, TbPersonFiling> implements IService<TbPersonFiling> {

    /**
     * 底层 Mapper 对象
     */
    @Autowired
    TbPersonFilingMapper tbPersonFilingMapper;
    @Resource
    @Lazy
    private TbTerminalService tbTerminalService;

    @Resource
    private UploadConfig uploadConfig;


    /**
     * 增
     */
    public void add(TbPersonFiling t) {
        String idCard = t.getIdCard();
        if (!IdcardUtil.isValidCard(idCard)) {
            throw new BusinessException("身份证不正确");
        }
        TbPersonFiling db = findByIdCardAndDeptId(idCard, t.getDeptId());
        if (db != null) {
            throw new BusinessException("人员已存在");
        }
        String pin = RandomUtil.randomNumbers(9);
        t.setPin(pin);
        this.handlerTime(t);
        t.setCreateBy(StpUtil.getLoginIdAsString()).setCreateTime(new Date());
        this.save(t);
        issued(t);
    }

    private void handlerTime(TbPersonFiling t) {
        t.setUpdateTime(new Date()).setUpdateBy(StpUtil.getLoginIdAsString());
    }

    /**
     * 删
     */
    public void delete(Integer id) {
        TbPersonFiling db = this.getById(id);
        deleteTerminalWhilte(db);
        this.removeById(id);
    }

    /**
     * 改
     */
    public void update(TbPersonFiling t) {
        String idCard = t.getIdCard();
        if (!IdcardUtil.isValidCard(idCard)) {
            throw new BusinessException("身份证不正确");
        }
        TbPersonFiling idCardDb = findByIdCardAndDeptId(idCard, t.getDeptId());
        if (idCardDb != null && !idCardDb.getId().equals(t.getId())) {
            throw new BusinessException("人员已存在");
        }
        TbPersonFiling db = this.getById(t.getId());
        handlerTime(t);
        //改了身份证或者名字,部门，要把旧的人员信息清除，下发新的
        if (!StrUtil.equals(t.getIdCard(), db.getIdCard())
                || !StrUtil.equals(t.getName(), db.getName())
                || !t.getDeptId().equals(db.getDeptId())
        ) {
            //删除旧的
            deleteTerminalWhilte(db);
            //下发新的
            issued(t);
        }
        this.updateById(t);
    }

    /**
     * 查
     */
    public TbPersonFiling getById(Integer id) {
        TbPersonFiling db = tbPersonFilingMapper.getById(id);

        return db;
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    public List<TbPersonFiling> getList(SoMap so) {
        return tbPersonFilingMapper.getList(so);
    }

    /**
     * 根据身份证查找
     *
     * @param idCard
     * @return
     */
    public TbPersonFiling findByIdCardAndDeptId(String idCard, Long deptId) {
        QueryWrapper<TbPersonFiling> ew = new QueryWrapper<>();
        ew.eq("id_card", idCard);
        ew.eq("dept_id", deptId);
        return getOne(ew);
    }

    /**
     * 下发单个白名单到所有设备
     *
     * @param filing
     */
    public void issued(TbPersonFiling filing) {
        List<TbTerminal> tbTerminalList = tbTerminalService.findByDeptId(filing.getDeptId());
        JSONObject params = builderParams(filing);
        tbTerminalList.forEach(tbTerminal -> {
            tbTerminalService.setCommand(CommandType.ISSUED_USER_COMMAND.getFuncId(), params, tbTerminal);
            tbTerminalService.startHeartCommand(tbTerminal.getSn());
        });
    }


    /**
     * 下发所有人员白名单到单个设备
     *
     * @param t
     */
    public void issued(TbTerminal t) {
        Long deptId = t.getDeptId();
        List<TbPersonFiling> list = this.findByDeptId(deptId);
        list.forEach(tbPersonFiling -> {
            JSONObject params = builderParams(tbPersonFiling);
            tbTerminalService.setCommand(CommandType.ISSUED_USER_COMMAND.getFuncId(), params, t);
        });
    }

    /**
     * 删除设备某个人的白名单
     *
     * @param t
     */
    public void deleteTerminalWhilte(TbPersonFiling t) {
        List<TbTerminal> tbTerminalList = tbTerminalService.findByDeptId(t.getDeptId());
        JSONObject params = new JSONObject();
        params.putIfAbsent("pin", Collections.singleton(t.getPin()));
        tbTerminalList.forEach(tbTerminal -> {
            tbTerminalService.setCommand(CommandType.DELETE_USER_COMMAND.getFuncId(), params, tbTerminal);
            tbTerminalService.startHeartCommand(tbTerminal.getSn());
        });

    }

    /**
     * 构建人员下发数据
     *
     * @param tbPersonFiling
     * @return
     */
    private JSONObject builderParams(TbPersonFiling tbPersonFiling) {
        WhitelistDTO dto = new WhitelistDTO();
        dto.setIdentity_number(tbPersonFiling.getIdCard())
                .setPin(tbPersonFiling.getPin())
                .setName(tbPersonFiling.getName());
        String photo = tbPersonFiling.getPhoto();
        if (StrUtil.isNotEmpty(photo)) {
            String filePath = new File(uploadConfig.rootFolder + tbPersonFiling.getPhoto().replace(SystemObject.config.getDomain(), "")).getAbsolutePath();
            dto.setPhoto(Base64Encoder.encode(FileUtil.readBytes(filePath)));
        }
        JSONObject params = new JSONObject();
        params.putIfAbsent("users", dto);
        return params;
    }


    /**
     * 查找部门的白名单
     *
     * @param deptId
     * @return
     */
    public List<TbPersonFiling> findByDeptId(Long deptId) {
        QueryWrapper<TbPersonFiling> ew = new QueryWrapper<>();
        ew.eq("dept_id", deptId);
        return list(ew);
    }

    public TbPersonFiling findByPin(String pin) {
        QueryWrapper<TbPersonFiling> ew = new QueryWrapper<>();
        ew.eq("pin", pin);
        return getOne(ew);
    }
}
