package com.pj.project.tb_person_visit_record;

import java.io.Serializable;
import java.util.Date;

import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import lombok.AllArgsConstructor;
import lombok.EqualsAndHashCode;

import lombok.Data;
import lombok.Getter;
import lombok.experimental.Accessors;

/**
 * Model: tb_person_visit_record -- 人员出入记录
 *
 * @author qzyReal
 */
@Data
@Accessors(chain = true)
@TableName(TbPersonVisitRecord.TABLE_NAME)
@EqualsAndHashCode(callSuper = false)
public class TbPersonVisitRecord extends Model<TbPersonVisitRecord> implements Serializable {

    // ---------- 模块常量 ----------
    /**
     * 序列化版本id
     */
    private static final long serialVersionUID = 1L;
    /**
     * 此模块对应的表名
     */
    public static final String TABLE_NAME = "tb_person_visit_record";
    /**
     * 此模块对应的权限码
     */
    public static final String PERMISSION_CODE = "tb-person-visit-record";
    public static final String PERMISSION_CODE_EDIT = "tb-person-visit-record-edit";
    public static final String PERMISSION_CODE_ADD = "tb-person-visit-record-add";
    public static final String PERMISSION_CODE_EXPORT = "tb-person-visit-record-export";


    // ---------- 表中字段 ----------
    /**
     * 主键
     */
    @TableId(type = IdType.AUTO)
    private Long id;

    /**
     * 出入(1=入场,2=离场)
     */
    private Integer direction;
    /**
     * [1=补录,2=设备上传]
     */
    private Integer type;

    /**
     * 部门ID
     */
    private Long deptId;



    /**
     * 姓名
     */
    private String name;
    private String phone;

    /**
     * 图片
     */
    private String picture;

    /**
     * 身份证
     */
    private String idCard;

    /**
     * 通过通道
     */
    private String channel;

    /**
     * 场所
     */
    private String venues;

    /**
     * 体温
     */
    private String temperature;

    /**
     * 健康码
     */
    private String healthStatus;

    /**
     * 上传设备
     */
    private String sn;
    private String terminalName;

    /**
     * 时间
     */
    private Date visitDate;

    /**
     * 创建时间
     */
    private Date createTime;
    private Date updateTime;
    private Boolean allowPass;

    /**
     * 部门名称
     */
    private String deptName;

    @TableField(exist = false)
    private String idCadStr;

    @TableField(exist = false)
    private String typeStr;
    @TableField(exist = false)
    private String directionStr;

    @Getter
    @AllArgsConstructor
    public static enum TypeEnum {
        UPLOAD(2, "设备上传"),
        ADD(1, "补录");

        private Integer type;
        private String desc;
    }

    @Getter
    @AllArgsConstructor
    public static enum DirectionEnum {
        OUT(2, "离场"),
        IN(1, "入场");

        private Integer direction;
        private String desc;
    }


    public String getIdCadStr() {
        String str = IdcardUtil.getBirthByIdCard(idCard);
        if (StrUtil.isNotEmpty(str)) {
            return idCard.replace(str, "********");
        }
        return idCard;
    }

    public String getTypeStr() {
        return type == TypeEnum.ADD.type ? "补录" : "设备上传";
    }

    public String getDirectionStr() {
        return direction == 1 ? "入场" : "离场";
    }
}
