package com.pj.project.tb_terminal;

import java.util.Date;
import java.util.List;
import java.util.Set;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.Update;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.pj.current.config.MyConfig;
import com.pj.current.global.BusinessException;
import com.pj.current.netty.command.constants.CommandType;
import com.pj.current.netty.dto.ParamsDTO;
import com.pj.current.netty.dto.RequestPackageDTO;
import com.pj.current.netty.dto.ResponsePackageDTO;
import com.pj.current.netty.heart.HeartBeatHandler;
import com.pj.current.satoken.StpUserUtil;
import com.pj.project.tb_command_log.TbCommandLog;
import com.pj.project.tb_command_log.TbCommandLogService;
import com.pj.project.tb_person_black.TbPersonBlackService;
import com.pj.project.tb_person_filing.TbPersonFilingService;
import com.pj.project.tb_venues.TbVenues;
import com.pj.project.tb_venues.TbVenuesService;
import com.pj.utils.cache.RedisUtil;
import io.netty.channel.ChannelHandlerContext;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.pj.utils.so.*;

import javax.annotation.Resource;

/**
 * Service: tb_terminal -- 设备管理
 *
 * @author qzyReal
 */
@Service
@Transactional(rollbackFor = Exception.class)
@Slf4j
public class TbTerminalService extends ServiceImpl<TbTerminalMapper, TbTerminal> implements IService<TbTerminal> {

    /**
     * 底层 Mapper 对象
     */
    @Autowired
    TbTerminalMapper tbTerminalMapper;
    @Resource
    private MyConfig myConfig;

    @Resource
    private TbCommandLogService tbCommandLogService;

    @Resource
    private TbVenuesService tbVenuesService;

    @Resource
    private TbPersonFilingService tbPersonFilingService;

    @Resource
    private TbPersonBlackService tbPersonBlackService;


    /**
     * 增
     */
    public void add(TbTerminal t) {
        this.save(t);
    }

    /**
     * 删
     */
    public void delete(Long id) {
        this.removeById(id);
    }

    /**
     * 改
     */
    public void update(TbTerminal t) {
        Long channelId = t.getChannelId();
        TbVenues tbVenues = tbVenuesService.getById(channelId);
        t.setChannelName(tbVenues.getName()).setDeptId(tbVenues.getDeptId())
                .setUpdateBy(StpUtil.getLoginIdAsString()).setUpdateTime(new Date());
        TbTerminal db = this.getById(t.getId());
        if (!t.getDeptId().equals(db.getDeptId())) {
            //更换组织，先清除所有数据再下发
            this.setCommand(CommandType.ISSUED_CLEAR_COMMAND.getFuncId(), null, t);
            //下发新的白名单
            tbPersonFilingService.issued(t);
            //下发新的黑名单
            tbPersonBlackService.issued(t);
            //开始设置心跳command=1
            startHeartCommand(t.getSn());
        }
        this.updateById(t);
    }


    /**
     * 查
     */
    TbTerminal getById(Long id) {
        return super.getById(id);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    public List<TbTerminal> getList(SoMap so) {
        return tbTerminalMapper.getList(so);
    }

    /**
     * 初始化设备
     *
     * @param sn
     * @param clientIp
     */
    public void initTerminal(String sn, String clientIp) {
        TbTerminal tbTerminal = findBySN(sn);
        if (tbTerminal == null) {
            tbTerminal = new TbTerminal();
            tbTerminal.setIpAddress(clientIp).setState(TbTerminal.StateType.ONLINE.getState())
                    .setSn(sn).setCreateTime(new Date())
                    .setDeptId(StpUserUtil.ADMIN_DEPT_ID);
            this.save(tbTerminal);
            return;
        }
        if (!StrUtil.equals(tbTerminal.getState(), TbTerminal.StateType.ONLINE.getState())) {
            tbTerminal.setState(TbTerminal.StateType.ONLINE.getState());
            this.updateById(tbTerminal);
        }
    }

    /**
     * 根据sn查找
     *
     * @param sn
     * @return
     */
    public TbTerminal findBySN(String sn) {
        QueryWrapper<TbTerminal> ew = new QueryWrapper<>();
        ew.eq("sn", sn);
        return getOne(ew);
    }

    /**
     * 重启
     *
     * @param sn
     */
    public void restart(String sn) {
        if (HeartBeatHandler.STATE_CACHE.get(sn) == null) {
            throw new BusinessException("设备已离线");
        }
        TbTerminal tbTerminal = this.findBySN(sn);
        //缓存命令
        setCommand(CommandType.ISSUED_REBOOT_COMMAND.getFuncId(), null, tbTerminal);
        //心跳缓存
        startHeartCommand(sn);
    }

    /**
     * 设置命令
     *
     * @param funcId func id
     * @param params 具体参数
     * @return
     */
    public void setCommand(String funcId, Object params, TbTerminal tbTerminal) {
        String sn = tbTerminal.getSn();
        String msgId = RandomUtil.randomNumbers(8);
        String key = myConfig.getCommandPrefix() + sn;
        String commandStr = RedisUtil.get(key);
        List<RequestPackageDTO> commandList = JSONUtil.toList(commandStr, RequestPackageDTO.class);
        RequestPackageDTO commandDTO = new RequestPackageDTO();
        RequestPackageDTO.PayloadBean payloadBean = new RequestPackageDTO.PayloadBean();

        payloadBean.setParams(params);
        commandDTO.setFuncId(funcId).setPayload(payloadBean)
                .setMsgId(msgId);
        commandList.add(commandDTO);
        TbCommandLog tbCommandLog = new TbCommandLog();
        tbCommandLog.setChannelName(tbTerminal.getChannelName()).setCommand(JSONUtil.toJsonStr(commandDTO))
                .setDeptId(tbTerminal.getDeptId()).setMsgId(msgId).setSn(sn)
                .setTerminalName(tbTerminal.getName()).setCreateTime(new Date()).setCreateBy(StpUserUtil.getAdminName());
        //日志记录
        tbCommandLogService.save(tbCommandLog);
        RedisUtil.set(myConfig.getCommandPrefix() + sn, JSONUtil.toJsonStr(commandList));
    }

    /**
     * 查看设备属性,不记录日志
     * @param funcId
     * @param sn
     */
    public void checkOptions(String funcId, String sn) {
        String msgId = RandomUtil.randomNumbers(8);
        String key = myConfig.getCommandPrefix() + sn;
        String commandStr = RedisUtil.get(key);
        List<RequestPackageDTO> commandList = JSONUtil.toList(commandStr, RequestPackageDTO.class);
        RequestPackageDTO commandDTO = new RequestPackageDTO();
        commandDTO.setFuncId(funcId)
                .setMsgId(msgId);
        commandList.add(commandDTO);
        RedisUtil.set(myConfig.getCommandPrefix() + sn, JSONUtil.toJsonStr(commandList));
    }

    /**
     * 设置心跳command=1,设备开始请求命令服务器接口
     *
     * @param sn sn码
     */
    public void startHeartCommand(String sn) {
        log.info("设置心跳命令===========:{}", sn);
        RequestPackageDTO requestPackageDTO = RequestPackageDTO.getHeartCommandDTO();
        RedisUtil.set(myConfig.getHeartPrefix() + sn, JSONUtil.toJsonStr(requestPackageDTO));
    }

    /**
     * 解绑设备
     *
     * @param id
     */
    public void unbind(Long id) {
        TbTerminal tbTerminal = this.getById(id);
        if (StrUtil.equals(tbTerminal.getState(),TbTerminal.StateType.OFFLINE.getState())){
            throw new BusinessException("设备已离线");
        }
        tbTerminalMapper.unbind(id);
    }

    /**
     * 清除数据
     *
     * @param id
     */
    public void clearData(Long id) {
        TbTerminal tbTerminal = this.getById(id);
        if (StrUtil.equals(tbTerminal.getState(),TbTerminal.StateType.OFFLINE.getState())){
            throw new BusinessException("设备已离线");
        }
        this.clearData(tbTerminal);
    }

    /**
     * 将离线设备设置为下线
     *
     * @param snList
     */
    public void offlineTerminal(List<String> snList) {
        UpdateWrapper<TbTerminal> ew = new UpdateWrapper<>();
        ew.notIn(!snList.isEmpty(), "sn", snList);
        ew.setSql("state='" + TbTerminal.StateType.OFFLINE.getState() + "'");
        this.update(ew);
    }

    /**
     * 根据部门id查找设备
     *
     * @param deptId
     * @return
     */
    public List<TbTerminal> findByDeptId(Long deptId) {
        QueryWrapper<TbTerminal> ew = new QueryWrapper<>();
        ew.eq("dept_id", deptId);
        return list(ew);
    }

    /**
     * 清空所有数据
     *
     * @param tbTerminal
     */
    public void clearData(TbTerminal tbTerminal) {
        setCommand(CommandType.ISSUED_CLEAR_COMMAND.getFuncId(), null, tbTerminal);
        startHeartCommand(tbTerminal.getSn());
    }

    /**
     * 远程开门
     * @param id
     */
    public void open(Long id) {
        TbTerminal tbTerminal = this.getById(id);
        if (StrUtil.equals(tbTerminal.getState(),TbTerminal.StateType.OFFLINE.getState())){
            throw new BusinessException("设备已离线");
        }
        setCommand(CommandType.ISSUED_NORMALOPEN_COMMAND.getFuncId(), null, tbTerminal);
        startHeartCommand(tbTerminal.getSn());
    }

    /**
     * 同步时间
     * @param id
     */
    public void asyncTime(Long id){
        TbTerminal tbTerminal = this.getById(id);
        if (StrUtil.equals(tbTerminal.getState(),TbTerminal.StateType.OFFLINE.getState())){
            throw new BusinessException("设备已离线");
        }
        JSONObject params=new JSONObject();
        params.set("time",System.currentTimeMillis()/1000);
        setCommand(CommandType.SYNC_TIME_COMMAND.getFuncId(), params, tbTerminal);
        startHeartCommand(tbTerminal.getSn());

    }
}
